﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using NHibernate;
using NHibernate.Linq;
using NHibernate.Criterion;
using UsingNHibernate.Models;
using Syncfusion.JavaScript;


namespace UsingNHibernate.Controllers
{
    public class HomeController : Controller
    {
        public IEnumerable<EmployeeMap> list { get; set; }

        public int Count { get; set; }

        public ActionResult Index()
        {
            return View();
        }
       
        //PROVIDE GRID DATASOURCE
        public ActionResult GetData(DataManager dm)
        {
            using (ISession session = AppSession.OpenSession())
            {
                list = ToQuery(dm,session).List<EmployeeMap>();
            }
            return Json(new { result = list, count = Count }, JsonRequestBehavior.AllowGet);
        }

        //PERFORM INSERT
        public ActionResult PerformInsert(string action, EmployeeMap value)
        {
            using (ISession session = AppSession.OpenSession())
            {
                using (ITransaction transaction = session.BeginTransaction())
                {
                    session.Save(value);
                    transaction.Commit();
                }
                list = session.Query<EmployeeMap>().ToList();
            }
            return Json(new { result = list, count = list.Count() }, JsonRequestBehavior.AllowGet);
        }

        //PERFORM UPDATE
        public ActionResult PerformUpdate(int key, EmployeeMap value)
        {
            using (ISession session = AppSession.OpenSession())
            {
                var employeetoUpdate = session.Get<EmployeeMap>(key);

                employeetoUpdate.Designation = value.Designation;
                employeetoUpdate.FirstName = value.FirstName;
                employeetoUpdate.LastName = value.LastName;

                using (ITransaction transaction = session.BeginTransaction())
                {
                    session.Save(employeetoUpdate);
                    transaction.Commit();
                }
                list = session.Query<EmployeeMap>().ToList();
            }

            return Json(new { result = list, count = list.Count() }, JsonRequestBehavior.AllowGet);
        }

        //PERFORM DELETE
        public ActionResult PerformDelete(int key)
        {
            using (ISession session = AppSession.OpenSession())
            {
                using (ITransaction transaction = session.BeginTransaction())
                {
                    session.Delete(session.Get<EmployeeMap>(key));
                    transaction.Commit();
                }
                list = session.Query<EmployeeMap>().ToList();
            }
            return Json(new { result = list, count = list.Count() }, JsonRequestBehavior.AllowGet);
        }

        private ICriteria ToQuery(DataManager dm, ISession session) {

            ICriteria criteria = session.CreateCriteria<EmployeeMap>();
            
            Count = session.CreateCriteria<EmployeeMap>().List().Count;
            
            Order order = null;       
            
            if (dm.Where != null && dm.Where.Count > 0) //Filtering
            {                
                 for (var i = 0; i < dm.Where.Count; i++) {

                     WhereFilter filter = dm.Where[i];
                     if (filter.IsComplex)
                         WhereFilter(criteria, filter);
                     if (filter.Operator == "equal")
                         criteria.Add(Restrictions.Eq(filter.Field, filter.value));
                     if (filter.Operator == "notequal")
                         criteria.Add(Restrictions.Not(Restrictions.Eq(filter.Field, filter.value)));
                     if (filter.Operator == "greaterthan")
                         criteria.Add(Restrictions.Ge(filter.Field, filter.value));
                     if (filter.Operator == "lessthan")
                         criteria.Add(Restrictions.Lt(filter.Field, filter.value));
                     if (filter.Operator == "greaterthanorequal")
                         criteria.Add(Restrictions.Ge(filter.Field, filter.value));
                     if (filter.Operator == "lessthanorequal")
                         criteria.Add(Restrictions.Le(filter.Field, filter.value));
                     if (filter.Operator == "startswith")
                         criteria.Add(Restrictions.Like(filter.Field, filter.value + "%"));
                     if (filter.Operator == "endswith")
                         criteria.Add(Restrictions.Like(filter.Field,  "%" + filter.value));
                     if (filter.Operator == "contains")
                         criteria.Add(Restrictions.Like(filter.Field, filter.value.ToString(), MatchMode.Anywhere));
                     Count = criteria.List().Count; //Required to show filter count
                }                
            }
           
            if (dm.Sorted != null && dm.Sorted.Count > 0) //Sorting
            {
                for (var i = 0; i < dm.Sorted.Count; i++) {                   
                    order =  dm.Sorted[i].Direction == "ascending" ? Order.Asc(dm.Sorted[i].Name) : Order.Desc(dm.Sorted[i].Name);
                    criteria.AddOrder(order);
                }
            }
            if (dm.Skip != 0)
            {
                criteria.SetFirstResult(dm.Skip);
            }
            if (dm.Take != 0)
            {
                criteria.SetMaxResults(dm.Take);
            }
            return criteria;
        }
        private ICriteria WhereFilter(ICriteria criteria, WhereFilter list)
        {

            for (var i = 0; i < list.predicates.Count; i++)
            {
                WhereFilter filter = list.predicates[i];
                if (filter.Operator == "equal")
                    criteria.Add(Restrictions.Eq(filter.Field, filter.value));
                if (filter.Operator == "notequal")
                    criteria.Add(Restrictions.Not(Restrictions.Eq(filter.Field, filter.value)));
                if (filter.Operator == "greaterthan")
                    criteria.Add(Restrictions.Ge(filter.Field, filter.value));
                if (filter.Operator == "lessthan")
                    criteria.Add(Restrictions.Lt(filter.Field, filter.value));
                if (filter.Operator == "greaterthanorequal")
                    criteria.Add(Restrictions.Ge(filter.Field, filter.value));
                if (filter.Operator == "lessthanorequal")
                    criteria.Add(Restrictions.Le(filter.Field, filter.value));
                if (filter.Operator == "startswith")
                    criteria.Add(Restrictions.Like(filter.Field, filter.value + "%"));
                if (filter.Operator == "endswith")
                    criteria.Add(Restrictions.Like(filter.Field, "%" + filter.value));
                if (filter.Operator == "contains")
                    criteria.Add(Restrictions.Like(filter.Field, filter.value.ToString(), MatchMode.Anywhere));
            }
            return criteria;
        }
    }
}